/*
 * name breakout.pde
 * date Sun Aug 04 22:40:31 JST 2013
 * 
 * Released under the 'Buy Me a Beer' license
 * (If we ever meet, you buy me a beer)
 */

import net.triring.emuduino.*;
import ddf.minim.AudioOutput;
// Pin割り当て
// constants won't change. They're used here to 
// set pin numbers:
int         D2 =   2;  // Switch:Push
int         D3 =   3;  // Switch:Push
int        BuzzPin =  10;  // Speaker:
int         PaddlePin =   5;  // Slider:INPUT:HORIZONTAL
int   S0 =   0;  // Serial
int   S1 =   1;  // Serial
int  S19 =  19;  // LedMatrix
int  S18 =  18;  // LedMatrix

// パッドの位置と長さの情報を管理するデータ群
int paddle_len = 3;
int paddle_pos = 3;

int Ray   =   8;
int Width  = Ray * 2 * 8;
int Height = Ray * 2 * 8;

int Row    = 8; //  行
int Column = 8; //  列

int INIT   = 0;
int WAITING  =  1;
int BEEP =  2;
int PLAY  = 3;

int SetupVolumeValue = 0;

int State = 0;
int Count = 0;
int Tick  = 0;
int Rnd   = 0;

int RIGHT  =  1;
int LEFT   = -1;
SysState sys;

Brick[][] brick = new Brick[8][8];
Paddle paddle;
Ball   ball;

int[][] gmap = {
  { 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0 },

  { 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0 },
  { 0, 0, 0, 0, 0, 0, 0, 0 }
};

void setup() {
  initComponents();
  // initialize
     pinMode(D2,  INPUT);  // Switch:Push
     pinMode(D3,  INPUT);  // Switch:Push
     pinMode(BuzzPin, OUTPUT);  // Speaker:

  sys = new SysState();
  State = sys.INIT_SYS;

  // ブロック位置の初期化
  for (int y = 0; y < 3; y += 1) {
    for (int x = 0; x < Column; x += 1) {
      brick[x][y] = new Brick(x, y, 8, #FF0000, Cell.ACTIVE);
      brick[x][y].setDeadColor(net.triring.emuduino.LedMatrix.OFF);
      brick[x][y].setAliveColor(net.triring.emuduino.LedMatrix.RED);
      brick[x][y].draw();
    }
  }

  // パドル位置の初期化
  paddle = new Paddle(3, 7, 8, net.triring.emuduino.LedMatrix.BLUE, Cell.ACTIVE, 3);
  paddle.draw();

  // ボール位置の初期化
  ball = new Ball(3, 4, 8, net.triring.emuduino.LedMatrix.GREEN, Cell.ACTIVE, 8, 8);
  ball.draw();
}
void init_system() {
  // ブロック位置の初期化
  for (int y = 0; y < 3; y += 1) {
    for (int x = 0; x < Column; x += 1) {
      brick[x][y].setLife(Cell.ACTIVE);
      brick[x][y].setDeadColor(net.triring.emuduino.LedMatrix.OFF);
      brick[x][y].setAliveColor(net.triring.emuduino.LedMatrix.RED);
      brick[x][y].draw();
    }
  }
  paddle.setLength(3);
  paddle.setLife(Cell.ACTIVE);
  paddle.setX(3);
  paddle.setY(7);

  ball.setLife(Cell.ACTIVE);
  ball.setX(3);
  ball.setY(4);
}

int getVolumeValue() {
  return analogRead(PaddlePin);
}

int getPddlePozzy() {
  int val = analogRead(PaddlePin); // Returns    int (0 to 1023) 
  int n = val / 86;
  int pos = 0;
  switch (n) {
    case  0 : pos = 5; beep(220, 16); break;
    case  1 : pos = 5; beep(220, 16); break;
    case  2 : pos = 5; beep(220, 16); break;
    case  3 : pos = 5; beep(220, 16); break;
    case  4 : pos = 5; break;
    case  5 : pos = 4; break;
    case  6 : pos = 3; break;
    case  7 : pos = 2; break;
    case  8 : pos = 1; break;
    case  9 : pos = 0; break;
    case 10 : pos = 0; beep(220, 16); break;
    case 11 : pos = 0; beep(220, 16); break;
    case 12 : pos = 0; beep(220, 16); break;
    case 13 : pos = 0; beep(220, 16); break;
  }
//  digitalWrite(BuzzPin, LOW );
  return pos;  
}

void beep(int note, int noteDuration) {
  tone(BuzzPin, note, noteDuration);
}

void loop() {
  int v = 0;
  int diff = 0;
        Tick++;
  paddle_pos = getPddlePozzy ();
  switch(State) {
    case INIT :
    v = getVolumeValue ();
    diff = v - SetupVolumeValue;
    if ((diff * diff) > 100) {
       State = BEEP;
    }   // Serial.println ("WAITING");
    break;
  case WAITING:
    v = getVolumeValue ();
    diff = v - SetupVolumeValue;
    if ((diff * diff) > 100) {
       State = BEEP;
    }   // Serial.println ("WAITING");
    break;
  case BEEP:
          beep(880, 500);
  // Serial.println ("BEEP");
    State = PLAY;
    break;
  case PLAY:
    clrscr();
    if (keyPressed) {
      switch(key) {
      case 'J': 
      case 'j': 
        paddle.moveTo(LEFT);  
        break;
      case 'K': 
      case 'k': 
        paddle.moveTo(RIGHT); 
        break;
      }
    }
    ball.move();
    int bx = ball.getX();
    int by = ball.getY();
    int count = 0;
    // ブロックとボールの衝突の検出
    for (int y = 0; y < 3; y += 1) {
      for (int x = 0; x < Column; x += 1) {
        brick[x][y].checkCollision(bx, by);
        if (Brick.DEAD == brick[x][y].getLife()) {
          count++;
        }
        brick[x][y].draw();
      }
    }
    println("count=" + count);
    if (count > 23) State = 3;
    // CONGRATS  3 congratulation ゲームクリア

    // パドルとボールの衝突の検出
    int pos = paddle.checkCollision(bx, by);
    ball.setXSpeed(1);
    ball.setYSpeed(1);
    switch (pos) {
    case  1 :  
      ball.setXDir(-1); 
      ball.setYDir(-1);
      break;
    case  2 :
      int rnd = (int)random(0, 6);
      switch (rnd) {
          case  1 : ball.setXDir(-1); break;
          case  2 : ball.setXDir( 1); break;
          default : ball.setXDir( 0); break;
      }
      ball.setYDir(-1);
      break;
    case  3 :  
      ball.setXDir( 1); 
      ball.setYDir(-1);
      break;
    }
    ball.draw();
    paddle.draw();
    break;
  case  3 : // CONGRATS  3 congratulation ゲームクリア
    if (keyPressed) {
      switch(key) {
      case 'J': 
      case 'j': 
      case 'K': 
      case 'k': 
        State = 0;  //PLAY
        init_system();
        break;
      }
    }
    break;
  case  4 : // GAMWEOVER 4 得点表示。何か押されたら，initへ
    break;
  }
}

void clrscr() {
  for (int y = 0; y < Row; y += 1) {
    for (int x = 0; x < Column; x += 1) {
          ledmatrix1.setPixel(x, y, net.triring.emuduino.LedMatrix.OFF);
      // fill(255, 255, 255);
      // ellipse(x * Ray * 2 + Ray, y * Ray * 2 + Ray, Ray * 2, Ray * 2);
    }
  }
}


void clearVram() {
  for (int i = 0; i < 8; i++) {
    for (int j = 0; j < 8; j++) {
      vram[i][j] = 0;
    }
  }
}

void setupBrick() {
  for (int y = 0; y < 3; y++) {
    for (int x = 0; x < 8; x++) {
      brick[x][y] = 1;
    }
  }
}

void drawBrick() {
  for (int y = 0; y < 3; y++) {
    for (int x = 0; x < 8; x++) {
      vram[x][y] = brick[x][y];
    }
  }
}

void drawPaddle(int paddle_pos) {
    for (int p = 0; p < paddle_len; p++) {
        vram[paddle_pos + p][7] = 1;
    }
}

void draw() {
  loop();
}

// システムの機能を完全に停止させる禁断のメソッド
// 決して、使ってはならない。
void balse() {
  System.exit(0);
}                                            

// digital INPUT
int digitalRead(int inPin) {
    int value = 0;
    switch (inPin) {
        case 2 : value = switch2.isState(); break;
        case 3 : value = switch3.isState(); break;
    }
    return value;
}
// analog INPUT
int analogRead(int inPin) {
    int value = 0;
    switch (inPin) {
        case   5 : value = slider5.getValue(); break;
    }
    return value;
}
// Speakerで使用する関数群

void tone(int pin, int frequency) {
  if (ToneFlag == frequency) return;
  ToneFlag = frequency;
  speaker10.setFrequency(frequency);
  sine.setFreq((float)frequency);
  auOut.unmute();
  auOut.sound();
}

void tone(int pin, int frequency, long duration) {
  if (ToneFlag == frequency) return;
  ToneFlag = frequency;
  sine.setFreq((float)frequency);
  auOut.unmute();
  auOut.sound();
  TimerTaskAudioStop timerTask = new TimerTaskAudioStop(); //タイマータスクインスタンス作成
  //タイマー開始
  DelayTimer.schedule(timerTask, duration);
}

void noTone(int pin) {
  ToneFlag = 0;
  speaker10.setFrequency(0);
  auOut.mute();
  auOut.noSound();
}
java.awt.Image readImage(String imgfile) {
  PImage pImage = loadImage(imgfile);
  return pImage.getImage();
}

//TimerTask内部クラス
class TimerTaskAudioStop extends java.util.TimerTask {
  public void run() {
    ToneFlag = 0;
    auOut.noSound();
    auOut.mute();
  }
}

// 指定された時間だけ停止する。単位はus
void delayMicroseconds(int t_us) {
  try {
    Thread.sleep(0, t_us * 1000);  // マイクロ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// 指定された時間だけ停止する。単位はms
void delay(long t_ms) {
  try {
    Thread.sleep(t_ms);  // ミリ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// arduino起動後の時間をus単位で返す。
long micros() {
  return (SystemStartTimeNS - System.nanoTime()) * 1000;
}

// 以下は、互換性のためだけに用意したダミー関数である。
void pinMode(int pin, int mode) {
   print("pin = " + pin + ", mode = " + mode);
}

/**
 * This method is called from within the constructor to initialize the form.
 * WARNING: Do NOT modify this code. The content of this method is always
 * regenerated by the Form Editor.
 */
// @SuppressWarnings("unchecked")
// <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
private void initComponents() {
  ToneFlag = 0;

  // digital
                   switch2 = new Switch(2 , "Push2",     0,    360);
                   switch3 = new Switch(3 , "Push3",  260,    360);
                 speaker10 = new Speaker(10 , "speaker10",  384,    0);
                   slider5 = new Slider(5 , "INPUT5",   0,  240, javax.swing.JSlider.HORIZONTAL);
                   slider5.setSize(384, 120);
  Serial = new SerialConsole(384, 240);
  ledmatrix1 = new LedMatrix(1, "LedMatrix 1", 8, 8, 80, 0);
  

  minim = new ddf.minim.Minim(this);
  // get a line out from Minim, default bufferSize is 1024, default sample rate is 44100, bit depth is 16
  auOut = minim.getLineOut(ddf.minim.Minim.STEREO);
  // create a sine wave Oscillator, set to 440 Hz, at 0.5 amplitude, sample rate from line out
  sine = new ddf.minim.signals.SineWave(440, 0.5, auOut.sampleRate());
  // set the portamento speed on the oscillator to 200 milliseconds
  sine.portamento(200);

  DelayTimer = new java.util.Timer(); //タイマーインスタンス作成

  // add the oscillator to the line out
  auOut.addSignal(sine);
  auOut.noSound();  // Disables all signals currently attached to this.
  auOut.mute();
  ToneFlag = 0;

  SystemStartTimeMS = System.currentTimeMillis();  //ミリ秒で表される現在の時間を返します。
  SystemStartTimeNS = System.nanoTime(); // 利用可能でもっとも正確なシステムタイマーの現在の値をナノ秒単位で返します。
  size(320, 128);
  setLocation(0, 600);
  background(15, 115, 145);
  fill( 15, 115, 145);  rect(0, 0, 320, 128);
  fill( 30, 160, 145);  textSize(60); text(TITLE , 24, 68);
  fill( 45, 192, 145);  textSize(60); text(TITLE , 22, 66);
  fill(255, 255, 255);  textSize(60); text(TITLE , 20, 64);
  fill(255, 255, 255);  textSize(18); text("Ver. " + Arduino.getVersion(), 120, 108);
  setLocation(0, 600);
}// </editor-fold>//GEN-END:initComponents

// 変数の宣言部
// Variables declaration - do not modify//GEN-BEGIN:variables

private String TITLE   = "Emuduino";
private String VERSION = "0.49"; // Arduino.javaで使用するプロパティファイル中のバージョンを参照する。

private long SystemStartTimeMS;
private long SystemStartTimeNS;

private static final int HIGH   = 0x1;
private static final int LOW    = 0x0;
private static final int INPUT  = 0x0;
private static final int OUTPUT = 0x1;

private int ToneFlag = 0;  // 0の場合は停止状態。再生中は，その周波数を保持

private java.util.Timer            DelayTimer;
private java.util.TimerTask        TimerTaskAudioStop;
private ddf.minim.Minim            minim; // http://code.compartmental.net/minim/javadoc/
private ddf.minim.AudioOutput      auOut;
private ddf.minim.signals.SineWave sine;

private Switch switch2;
private Switch switch3;
private Speaker speaker10;
private Slider slider5;
private SerialConsole Serial;
private LedMatrix ledmatrix1;


// End of variables declaration//GEN-END:variables


