/*
 * name LedCubeHelloWorld.pde
 * date Tue Feb 05 16:48:00 JST 2013
 * 
 * Released under the 'Buy Me a Beer' license
 * (If we ever meet, you buy me a beer)
 */

import net.triring.emuduino.*;
import ddf.minim.AudioOutput;
// Pin割り当て
// constants won't change. They're used here to 
// set pin numbers:

int Count = 0;
int Color = 0;

int    pat_before = 999;
float posTime = 0;
float runTime = 0;
int    lastTime = 0;

// 1パターンごとの表示時間(秒)
final static int PERIOD  = 10;

final static int  HEIGHT =  3; // 高さ(z軸)
final static int  DEPTH  =  3; // 奥行(y軸)
final static int  WIDTH   =  3; // 幅  (x軸)

// final static int  HEIGHT =  5; // 高さ(z軸)
// final static int  DEPTH  =  5; // 奥行(y軸)
// final static int  WIDTH   =  5; // 幅  (x軸)


final static int  ALL = HEIGHT * DEPTH * WIDTH;

  // twistの外周部分の縦列番号
  int xouter[] = new int[(WIDTH + DEPTH - 2) * 2];
  int youter[] = new int[(WIDTH + DEPTH - 2) * 2];

void setup() {
  // 3x3x3 の Led Cube で以下のパターンを繰り返して表示します。
  // 1個ずつ追加,波,平面,ツイスト
  initComponents();
// ledcube1.setFrameFileName("LedCube#####.png");
}

void loop() {
  int pat;
  int dt;
  boolean rst = false;

  //経過時間よりパターン決定
  pat = (millis() / 1000  / PERIOD) % 5;
  dt = getTimeSinceLastFrame();

  if (pat != pat_before) {
    //パターン切り替え時の処理
    lightNone();  
    delay(100);
    pat_before = pat;
    rst = true;
  }
  
   switch(pat) {
    case 4:
   // ledcube1.setCaptureMode(false);
      twist(dt, rst);  // ツイスト
      break;
    case 3:
   // ledcube1.setCaptureMode(true);
      panel(dt, rst); // 平面
      break;
    case 2:
   // ledcube1.setCaptureMode(true);
      wave(dt, rst);      // 波
      break;
    case 1:
   // ledcube1.setCaptureMode(false);
      increment(dt, rst); // 1個ずつ追加
      break;
    default:
   // ledcube1.setCaptureMode(false);
      lightAll();         //全点灯
  }
}

/**
 * ツイスト
 */
void twist(int dt, boolean rst) {
  int num = (WIDTH + DEPTH - 2) * 2;
  int x;
  int y;
  int i = 0;
  if (rst) {
    lightNone();
    posTime = 0;
    //CW
    for(x = 0 ; x < WIDTH - 1; x++) {
      xouter[i] = x;
      youter[i] = 0;
      i++;
    }
    for(y = 0 ; y < DEPTH - 1; y++) {
      xouter[i] = WIDTH - 1;
      youter[i] = y;
      i++;
    }
    for(x = WIDTH - 1 ; x > 0; x--) {
      xouter[i] = x;
      youter[i] = DEPTH - 1;
      i++;
    }
    for(y = DEPTH - 1 ; y > 0; y--) {
      xouter[i] = 0;
      youter[i] = y;
      i++;
    }   
  } else {
//  posTime += 1.0*dt / 1000 / 50;
    posTime += 1.0*dt / 1000;
  }
  int pos = ((int) posTime) % num;
  int  zz;
  for(int  z = 0; z < HEIGHT; z ++) { 
    zz = (pos + z) % num; 
    for(y = 0 ; y < DEPTH ; y++) {
      for(x = 0 ; x < WIDTH ; x++) {
        if (y == xouter[zz] && x == youter[zz]) {
           ledcube1.setColor(z, y, x, 0x00000000);
        } else {
           ledcube1.setColor(z, y, x, 0xffff0000);
        }
      }
    }
  }
}

/**
 * 平面
 */
void panel(int dt, boolean rst)
{
  if (rst) {
    posTime = 0;
  } else {
 // posTime += 1.0*dt / 1000 / 50;
    posTime += 1.0*dt / 1000;
  }
  int pos = ((int) posTime * 10000) % HEIGHT ;
  for(int  z = 0; z < HEIGHT; z ++) { 
    for(int  y = 0 ; y < DEPTH ; y++) {
      for(int  x = 0 ; x < WIDTH ; x++) {
        if (pos == z) {
           ledcube1.setColor(z, y, x, 0xffff0000);
        } else {
           ledcube1.setColor(z, y, x, 0x00000000);
        }
      }
    }
  }
}


/**
 * 波
 */
void wave(int dt, boolean rst)
{
  int half_width = int(0.5 * WIDTH);
  int half_depth = int(0.5 * DEPTH);
  float xfloat;
  float yfloat;
  float zfloat;
  int zint;
  if (rst) {
    runTime = 0;
  } else {  
    runTime += 1.0*dt / 1000;
  }
  for(int  y = 0 ; y < DEPTH ; y++) {
    yfloat = (y - half_depth);
    for(int  x = 0 ; x < WIDTH ; x++) {
      xfloat = (x - half_width);
      zfloat = (cos(xfloat * xfloat + yfloat * yfloat - runTime * 10) + 1) * .5 * HEIGHT;
      zint = constrain(int(zfloat + 0.5), 0, HEIGHT - 1);
      for (int  z = 0 ; z < HEIGHT; z++) {
        if (z == zint) {
           ledcube1.setColor(z, y, x, 0xffff0000);
        } else {
           ledcube1.setColor(z, y, x, 0x00000000);
        }
      }
    }
  }
}


/**
 * 一つづづ追加
 */
void increment(int dt, boolean rst)
{
  if (rst) {
    posTime = 0;
  } else {
    posTime += (ALL + 1.0) / PERIOD * dt / 1000;
  }
  int pos = ((int) posTime) % (ALL + 1);
  if (pos == ALL) {
    lightNone();
  } else {
    int  x = pos % WIDTH;
    int  y = ((pos - x) / WIDTH % DEPTH);
    int  z = (pos - y * WIDTH - x) / (WIDTH * DEPTH);  
    ledcube1.setColor(z, y, x, 0xffff0000);
  }
}

/**
 * 全点灯
 */
void lightAll()
{
  for (byte z = 0 ; z < HEIGHT; z++) {
    for (byte y = 0 ; y < DEPTH; y++) {
      for (byte x = 0 ; x < WIDTH; x++) {
        ledcube1.setColor(z, y, x, 0xffff0000);
      }
    }
  }
}



/**
 * 全消灯
 */
void lightNone()
{
  for (byte z = 0 ; z < HEIGHT; z++) {
    for (byte y = 0 ; y < DEPTH; y++) {
      for (byte x = 0 ; x < WIDTH; x++) {
        ledcube1.setColor(z, y, x, 0x00000000);
      }
    }
  }
}


void draw() {
  loop();
}


/**
 * 最後にこの関数を呼んでからの時間(milliseconds)を返す
 *
 * @return int
 */
int getTimeSinceLastFrame() {
  int dt = millis() - lastTime;
  lastTime = millis();
  return dt;
}


/* 以上 */


// システムの機能を完全に停止させる禁断のバルスメソッド
// 決して、使ってはならない。
void Balse() {
  System.exit(0);
}                                            

// digital OUTPUT
void digitalWrite(int outPin, int value) {
}

java.awt.Image readImage(String imgfile) {
  PImage pImage = loadImage(imgfile);
  return pImage.getImage();
}

//TimerTask内部クラス
class TimerTaskAudioStop extends java.util.TimerTask {
  public void run() {
    ToneFlag = 0;
    auOut.noSound();
    auOut.mute();
  }
}

// 指定された時間だけ停止する。単位はus
void delayMicroseconds(int t_us) {
  try {
    Thread.sleep(0, t_us * 1000);  // マイクロ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// 指定された時間だけ停止する。単位はms
void delay(long t_ms) {
  try {
    Thread.sleep(t_ms);  // ミリ秒単位の停止
  } 
  catch (InterruptedException e) {
  }
}

// arduino起動後の時間をus単位で返す。
long micros() {
  return (SystemStartTimeNS - System.nanoTime()) * 1000;
}

// 以下は、互換性のためだけに用意したダミー関数である。
void pinMode(int pin, int mode) {
   print("pin = " + pin + ", mode = " + mode);
}

/**
 * This method is called from within the constructor to initialize the form.
 * WARNING: Do NOT modify this code. The content of this method is always
 * regenerated by the Form Editor.
 */
// @SuppressWarnings("unchecked")
// <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
private void initComponents() {
  ToneFlag = 0;

  // digital
//  ledcube1  = new LedCube();
  ledcube1 = new LedCube(1, "LedCube Hello", HEIGHT, DEPTH, WIDTH, 0, 0);

  minim = new ddf.minim.Minim(this);
  // get a line out from Minim, default bufferSize is 1024, default sample rate is 44100, bit depth is 16
  auOut = minim.getLineOut(ddf.minim.Minim.STEREO);
  // create a sine wave Oscillator, set to 440 Hz, at 0.5 amplitude, sample rate from line out
  sine = new ddf.minim.signals.SineWave(440, 0.5, auOut.sampleRate());
  // set the portamento speed on the oscillator to 200 milliseconds
  sine.portamento(200);

  DelayTimer = new java.util.Timer(); //タイマーインスタンス作成

  // add the oscillator to the line out
  auOut.addSignal(sine);
  auOut.noSound();  // Disables all signals currently attached to this.
  auOut.mute();
  ToneFlag = 0;

  SystemStartTimeMS = System.currentTimeMillis();  //ミリ秒で表される現在の時間を返します。
  SystemStartTimeNS = System.nanoTime(); // 利用可能でもっとも正確なシステムタイマーの現在の値をナノ秒単位で返します。
  size(320, 128);
  setLocation(0, 600);
  background(15, 115, 145);
  fill( 15, 115, 145);  rect(0, 0, 320, 128);
  fill( 30, 160, 145);  textSize(60); text(TITLE , 24, 68);
  fill( 45, 192, 145);  textSize(60); text(TITLE , 22, 66);
  fill(255, 255, 255);  textSize(60); text(TITLE , 20, 64);
  fill(255, 255, 255);  textSize(18); text("Ver. " + Arduino.getVersion(), 120, 108);
  setLocation(0, 600);
}// </editor-fold>//GEN-END:initComponents

// 変数の宣言部
// Variables declaration - do not modify//GEN-BEGIN:variables

private String TITLE   = "Emuduino";
private String VERSION = "0.49"; // Arduino.javaで使用するプロパティファイル中のバージョンを参照する。

private long SystemStartTimeMS;
private long SystemStartTimeNS;

private static final int HIGH   = 0x1;
private static final int LOW    = 0x0;
private static final int INPUT  = 0x0;
private static final int OUTPUT = 0x1;

private int ToneFlag = 0;  // 0の場合は停止状態。再生中は，その周波数を保持

private java.util.Timer            DelayTimer;
private java.util.TimerTask        TimerTaskAudioStop;
private ddf.minim.Minim            minim; // http://code.compartmental.net/minim/javadoc/
private ddf.minim.AudioOutput      auOut;
private ddf.minim.signals.SineWave sine;

private LedCube ledcube1;

// End of variables declaration//GEN-END:variables
